@@ -2,9 +2,9 @@ package App::Prove;
 
 use strict;
 use TAP::Harness;
-use File::Find;
 use File::Spec;
 use Getopt::Long;
+use App::Prove::State;
 use Carp;
 
 use vars qw($VERSION);
@@ -15,11 +15,11 @@ App::Prove - Implements the C<prove> command.
 
 =head1 VERSION
 
-Version 3.02
+Version 3.04
 
 =cut
 
-$VERSION = '3.02';
+$VERSION = '3.04';
 
 =head1 DESCRIPTION
 
@@ -37,8 +37,9 @@ wrapper around an instance of this module.
 
 =cut
 
-my $IS_WIN32 = ( $^O =~ /^(MS)?Win32$/ );
-my $NEED_GLOB = $IS_WIN32;
+use constant IS_WIN32 => ( $^O =~ /^(MS)?Win32$/ );
+use constant IS_VMS => $^O eq 'VMS';
+use constant STATE_FILE => ( IS_WIN32 || IS_VMS ) ? '_prove' : '.prove';
 
 use constant PLUGINS => 'App::Prove::Plugin';
 
@@ -47,9 +48,10 @@ my @ATTR;
 BEGIN {
     @ATTR = qw(
       archive argv blib color directives exec failures fork formatter
-      harness includes modules plugins jobs lib merge parse quiet really_quiet recurse
-      backwards shuffle taint_fail taint_warn timer verbose
-      warnings_fail warnings_warn show_help show_man show_version
+      harness includes modules plugins jobs lib merge parse quiet
+      really_quiet recurse backwards shuffle taint_fail taint_warn timer
+      verbose warnings_fail warnings_warn show_help show_man
+      show_version test_args state
     );
     for my $attr (@ATTR) {
         no strict 'refs';
@@ -77,10 +79,13 @@ sub new {
     my $args = shift || {};
 
     my $self = bless {
-        argv     => [],
-        includes => [],
-        modules  => [],
-        plugins  => [],
+        argv          => [],
+        includes      => [],
+        modules       => [],
+        state         => [],
+        plugins       => [],
+        harness_class => 'TAP::Harness',
+        _state        => App::Prove::State->new( { store => STATE_FILE } ),
     }, $class;
 
     for my $attr (@ATTR) {
@@ -107,6 +112,12 @@ Dies on invalid arguments.
 sub process_args {
     my ( $self, @args ) = @_;
 
+    if ( defined( my $stop_at = _first_pos( '::', @args ) ) ) {
+        my @test_args = splice @args, $stop_at;
+        shift @test_args;
+        $self->{test_args} = \@test_args;
+    }
+
     if ( my @bad = map {"-$_"} grep {/^-(man|help)$/} @args ) {
         die "Long options should be written with two dashes: ",
           join( ', ', @bad ), "\n";
@@ -141,6 +152,7 @@ sub process_args {
             'I=s@'        => $self->{includes},
             'M=s@'        => $self->{modules},
             'P=s@'        => $self->{plugins},
+            'state=s@'    => $self->{state},
             'directives'  => \$self->{directives},
             'h|help|?'    => \$self->{show_help},
             'H|man'       => \$self->{show_man},
@@ -161,6 +173,14 @@ sub process_args {
     return;
 }
 
+sub _first_pos {
+    my $want = shift;
+    for ( 0 .. $#_ ) {
+        return $_ if $_[$_] eq $want;
+    }
+    return;
+}
+
 sub _exit { exit( $_[1] || 0 ) }
 
 sub _help {
@@ -181,14 +201,12 @@ sub _help {
 sub _color_default {
     my $self = shift;
 
-    return -t STDOUT && !$IS_WIN32;
+    return -t STDOUT && !IS_WIN32;
 }
 
 sub _get_args {
     my $self = shift;
 
-    $self->{harness_class} = 'TAP::Harness';
-
     my %args;
 
     if ( defined $self->color ? $self->color : $self->_color_default ) {
@@ -251,6 +269,10 @@ sub _get_args {
     $args{exec} = [ split( /\s+/, $self->exec ) ]
       if ( defined( $self->exec ) );
 
+    if ( defined( my $test_args = $self->test_args ) ) {
+        $args{test_args} = $test_args;
+    }
+
     return ( \%args, $self->{harness_class} );
 }
 
@@ -324,7 +346,12 @@ sub run {
         $self->_load_extensions( $self->modules );
         $self->_load_extensions( $self->plugins, PLUGINS );
 
-        my @tests = $self->_get_tests( @{ $self->argv } );
+        my $state = $self->{_state};
+        if ( defined( my $state_switch = $self->state ) ) {
+            $state->apply_switch(@$state_switch);
+        }
+
+        my @tests = $state->get_tests( $self->recurse, @{ $self->argv } );
 
         $self->_shuffle(@tests) if $self->shuffle;
         @tests = reverse @tests if $self->backwards;
@@ -337,7 +364,14 @@ sub run {
 
 sub _runtests {
     my ( $self, $args, $harness_class, @tests ) = @_;
-    my $harness    = $harness_class->new($args);
+    my $harness = $harness_class->new($args);
+
+    $harness->callback(
+        after_test => sub {
+            $self->{_state}->observe_test(@_);
+        }
+    );
+
     my $aggregator = $harness->runtests(@tests);
 
     $self->_exit( $aggregator->has_problems ? 1 : 0 );
@@ -386,63 +420,6 @@ sub _get_lib {
     return @libs ? \@libs : ();
 }
 
-sub _get_tests {
-    my $self = shift;
-    my @argv = @_;
-    my ( @tests, %tests );
-
-    unless (@argv) {
-        croak "No tests named and 't' directory not found"
-          unless -d 't';
-        @argv = 't';
-    }
-
-    # Do globbing on Win32.
-    if ($NEED_GLOB) {
-        @argv = map { glob "$_" } @argv;
-    }
-
-    foreach my $arg (@argv) {
-        if ( '-' eq $arg ) {
-            push @argv => <STDIN>;
-            chomp(@argv);
-            next;
-        }
-
-        if ( -d $arg ) {
-            my @files = $self->_expand_dir($arg);
-            foreach my $file (@files) {
-                push @tests => $file unless exists $tests{$file};
-            }
-            @tests{@files} = (1) x @files;
-        }
-        else {
-            push @tests => $arg unless exists $tests{$arg};
-            $tests{$arg} = 1;
-        }
-    }
-    return @tests;
-}
-
-sub _expand_dir {
-    my $self = shift;
-    my $dir  = shift;
-    my @tests;
-    if ( $self->recurse ) {
-        find(
-            {   follow => 1,    #21938
-                wanted =>
-                  sub { -f && /\.t$/ && push @tests => $File::Find::name }
-            },
-            $dir
-        );
-    }
-    else {
-        @tests = glob( File::Spec->catfile( $dir, '*.t' ) );
-    }
-    return sort @tests;
-}
-
 sub _shuffle {
     my $self = shift;
 
@@ -555,10 +532,14 @@ calling C<run>.
 
 =item C<shuffle>
 
+=item C<state>
+
 =item C<taint_fail>
 
 =item C<taint_warn>
 
+=item C<test_args>
+
 =item C<timer>
 
 =item C<verbose>
